#===========================================================================
# Information about a selection or window.
#===========================================================================
proc wordCount {} {
	if {[set chars [expr {[selEnd] - [getPos]}]]} {
		set lines [expr {[lindex [posToRowCol [selEnd]] 0] - [lindex [posToRowCol [getPos]] 0]}]
		set text [getSelect]
	} else {
		set chars [maxPos]
		set lines [lindex [posToRowCol $chars] 0]
		set text [getText 0 [maxPos]]
	}
	if {[regsub -all {[!=;.,\(\#\=\):\{\"\}]} $text " " ret]} {
		set words [llength $ret]
	} else {
		set words [llength $text]
	}
	alertnote [format "%d chars, %d words, %d lines" $chars $words $lines]
}

proc matchingLines {} {
	if [catch {prompt "Regular expression:" ""} reg] return
	if {![string length $reg]} return
	set reg ^.*$reg.*$
	set pos [getPos]
	set matches 0
	while {![catch {search -f 1 -r 1 -m 0 -i 1 $reg $pos} mtch]} {
		append lines "\r" [format "%4d: " [lindex [posToRowCol [lindex $mtch 0]] 0]] [eval getText $mtch]
		set pos [lindex $mtch 1]
		incr matches
	}
	new
	insertText [format "%d matching lines\r-----" $matches] $lines "\r"
}


#=============================================================================
# Random functions.
#=============================================================================

#***********************************************************************
#                                                                      *
#   Comment box and uncomment box courtesy of Igor Mikolic-Torreira.   *
#                                                                      *
#**********************************************************************/

proc commentBox {} {
# Set what the comment block will look like

	if {[getPos] == [selEnd]} {
		alertnote "Must select region to be commented."
		return
	}
	watchCursor
	set begComment "/*"
	set begComLen 2
	set endComment "*/"
	set endComLen 2
	set fillChar "*"
	set spaceOffset 3
	set aSpace " "

# First make sure we grab a full block of lines and adjust highlight

	set start [getPos]
	set start [lineStart $start]
	set end [selEnd]
	set end [nextLineStart [expr $end-1]]
	set text [getText $start $end]
	select $start $end

# Next turn it into a list of lines--possibly drop an empty 'last line'

	set lineList [split $text "\r"]
	set emptyLine [lsearch $lineList {}]
	if { $emptyLine != -1 } then {
		set numLines [llength $lineList]
		set lineList [lrange $lineList 0 [expr $numLines-2]]
	}
	set numLines [llength $lineList]
	
# Find the longest line length and determine the new line length

	set maxLength 0
	foreach thisLine $lineList {
		set thisLength [string length $thisLine]
		if { $thisLength > $maxLength } then { 
			set maxLength $thisLength 
		}
	}
	set newLength [expr {$maxLength + 2 + 2*$spaceOffset}]
	
# Now create the top & bottom bars and a blank line

	set topBar $begComment
	for { set i 0 } { $i < [expr {$newLength - $begComLen}] } { incr i } {
		set topBar $topBar$fillChar
	}
	set botBar ""
	for { set i 0 } { $i < [expr {$newLength - $endComLen}] } { incr i } {
		set botBar $botBar$fillChar
	}
	set botBar $botBar$endComment
	set blankLine $fillChar
	for { set i 0 } { $i < [expr {$newLength - 2}] } { incr i } {
		set blankLine $blankLine$aSpace
	}
	set blankLine $blankLine$fillChar
	
# For each line add stuff on left and spaces and stuff on right for box sides
# and concatenate everything into 'text'.  Start with topBar; end with botBar

	set text $topBar\r$blankLine\r
	
	set frontStuff $fillChar
	set backStuff $fillChar
	for { set i 0 } { $i < $spaceOffset } { incr i } {
		set frontStuff $frontStuff$aSpace  
		set backStuff $aSpace$backStuff
	}
	set backStuffLen [string length $backStuff]
	
	for { set i 0 } { $i < $numLines } { incr i } {
		set thisLine [lindex $lineList $i ]
		set thisLine $frontStuff$thisLine
		set thisLength [string length $thisLine]
		set howMuchPad [expr {$newLength - $thisLength - $backStuffLen}]
		for { set j 0 } { $j < $howMuchPad } { incr j } {
			set thisLine $thisLine$aSpace 
		}
		set thisLine $thisLine$backStuff
		set text $text$thisLine\r
	}
	
	set text $text$blankLine\r$botBar\r
	
# Now replace the old stuff

	replaceText	$start $end	$text
	select $start [expr {$start+[string	length $text]}]
}



proc uncommentBox {} {

	if {[getPos] == [selEnd]} {
		alertnote "Must select region to be uncommented."
		return
	}

# Set what the comment block will look like
	watchCursor
	set begComment "/*"
	set endComment "*/"
	set fillChar "*"
	set aSpace " "
	set aTab \t

# First make sure we grab a full block of lines

	set start [getPos]
	set start [lineStart $start]
	set end [selEnd]
	set end [nextLineStart [expr $end-1]]
	set text [getText $start $end]

# Make sure we're at the start and end of the box

	set startOK [string first $begComment $text]
	set endOK [string last $endComment $text]
	set textLength [string length $text]
	if { $startOK != 0 && $endOK != [expr {$textLength-3}] } then {
		alertnote "You must highlight the entire comment box, including the borders."
		return
	}
	
# Next turn it into a list of lines--possibly drop an empty 'last line'

	set lineList [split $text "\r"]
	set emptyLine [lsearch $lineList {}]
	if { $emptyLine != -1 } then {
		set numLines [llength $lineList]
		set lineList [lrange $lineList 0 [expr $numLines-2]]
	}
	set numLines [llength $lineList]
	
# Delete the first and last lines, recompute number of lines

	set lineList [lreplace $lineList [expr $numLines-1] [expr $numLines-1] ]
	set lineList [lreplace $lineList 0 0 ]
	set numLines [llength $lineList]
	
# Eliminate 2nd and 2nd-to-last lines if they are empty

	set eliminate $fillChar$aSpace$aTab
	set thisLine [lindex $lineList [expr $numLines-1]]
	set thisLine [string trim $thisLine $eliminate]
	if { [string length $thisLine] == 0 } then {
		set lineList [lreplace $lineList [expr $numLines-1] [expr $numLines-1] ]
	}
	set thisLine [lindex $lineList 0]
	set thisLine [string trim $thisLine $eliminate]
	if { [string length $thisLine] == 0 } then {
		set lineList [lreplace $lineList 0 0 ]
	}
	set numLines [llength $lineList]	
	
# For each line trim stuff on left and spaces and stuff on right

	for { set i 0 } { $i < $numLines } { incr i } {
		set thisLine [lindex $lineList $i]
		set thisLine [string trim $thisLine $eliminate]
		set lineList [lreplace $lineList $i $i $thisLine]
	}
	
# Now splice the lines back together, separating by carriage return...

	set text [join $lineList "\r"]
	set text $text\r
	
# Now replace the old stuff

	replaceText	$start $end	$text
	select $start [expr {$start+[string	length $text]}]
}
#================================================================================

proc transposeWords {} {
        forwardWord
        setMark
        backwardWord
        cut
        deleteChar
        forwardWord
        insertText "\ "
        paste
}

proc transposeChars {} {
        setMark
        forwardChar
        cut
        backwardChar
        paste
        forwardChar
}

proc nextFunc {} {
	searchFunc 1
}

proc prevFunc {} {
	searchFunc 0
}

proc searchFunc {dir} {
	global funcExpr
	set pos [getPos]
	select $pos
	if ($dir==1) {
		incr pos
	} else {
		set pos [expr $pos-1]
	}
	if {![catch {search -f $dir -i 1 -r 1 $funcExpr $pos} res]} {
		eval select $res
	}
}

#===========================================================================
# Comment routines.
#===========================================================================
proc commentPara {} {
}



#===========================================================================
# Sorting the selection.
# AUTHOR: David C. Black	 black@mpd.tandem.com
#===========================================================================
proc sortLines {} {
	set start [getPos]
	set end  [selEnd]
	if {$start == $end} {
		alertnote "You must highlight the section you wish to sort."
		return
	}
	if {[lookAt [expr $end-1]] != "\r"} {
		alertnote "The selection must consist only of complete lines."
		return
	}
	set text [getText $start [expr {$end-1}]]
	set text [join [lsort [split $text "\r"]] "\r"]
	replaceText $start [expr {$end-1}] $text
	select $start $end
}



proc compareWindows {} {
    set one [listpick [lsort [winNames -f]]]
    set two [listpick [lsort [winNames -f]]]
	compare-windows $one $two
}


#===========================================================================
# Dump all current settings into a file.
#===========================================================================
proc insertGlobalSettings {} {
	uplevel #0 {
		foreach var [info globals] {
			if {![catch {set $var}]} {
				insertText "set " $var " \{" [set $var] "\}\r"
			}
		}
	}
}


#================================================================================
# Substitute global variables in possibly nested list.
#================================================================================
proc subVars {words} {
	global silly
	global a
	set silly $words
	set out {}
	foreach a $words {
		if {[llength $a] == 1} {
			lappend out [uplevel #0 {eval set x $a}]
		} else {
			lappend out [subVars $a]
		}
	}
	return $out
}

